using Godot;
using System;

public class WorldGrid : Node2D
{
    [Export]
    private PackedScene TileScene { get; set; }

    [Export]
    public int Size { get; set; }
    
    [Export]
    public int CellSize { get; set; }
    
    private Tile[,] _tiles;

    // Called when the node enters the scene tree for the first time.
    public override void _Ready()
    {
        GenerateGrid(Size);
    }

    public override void _Process(float delta)
    {
        base._Process(delta);
        
        for (int x = 0; x < Size; x++)
        {
            for (int y = 0; y < Size; y++)
            {
                var tile = _tiles[x, y];
                var material = tile.material;
                material.SetShaderParam("isHighlighted", tile.isHighlighted ? 1 : 0);
            }
        }
    }
    
    public bool IsInBounds(int x, int y)
    {
        return x >= 0 && x < Size && y >= 0 && y < Size;
    }

    public void ToggleTileHighlight(int x, int y)
    {
        if (!IsInBounds(x, y))
            return;

        _tiles[x, y].isHighlighted ^= true;
    }
    
    public void SetTileValue(int x, int y, float value)
    {
        _tiles[x, y].value = value;
    }

    public void GetGridPos(Vector2 position, out int x, out int y)
    {
        x = Mathf.FloorToInt(position.x / CellSize);
        y = Mathf.FloorToInt(position.y / CellSize);
    }

    private void Clear()
    {
        _tiles = null;

        int children = this.GetChildCount();
        GD.Print(children);
        for (int i = 0; i < children; i++)
        {
            var child = this.GetChild(i);
            child.QueueFree();
        }
    }

    private void GenerateGrid(int size)
    {
        this.Clear();
        
        _tiles = new Tile[size, size];

        for (int x = 0; x < size; x++)
        {
            for (int y = 0; y < size; y++)
            {
                var tile = new Tile();
                tile.isHighlighted = false;
                tile.value = 0.0f;
                
                var node = TileScene.Instance<Node2D>();
                this.AddChild(node);
                var position = new Vector2
                {
                    x = (x + .5f) * CellSize,
                    y = (y + .5f) * CellSize
                };
                node.Position = position;
                var canvasItem = (CanvasItem)node;
                tile.material = (ShaderMaterial)canvasItem.Material;
                
                _tiles[x, y] = tile;
            }
        }
    }
}
